/*
 * (c) 2022, SWD Embedded Systems Limited, http://www.kpda.ru
 */

#ifndef PROTO_H_
#define PROTO_H_


#include <stdio.h>
#include <stdint.h>
//#include <i86.h>

#include <sys/neutrino.h>
#include <sys/mman.h>
#include <pthread.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>

#include <hw/inout.h>
#include <hw/i2c.h>
#include <stdlib.h>
#include <unistd.h>

#include <time.h>
#include <hw/pci.h>
#include <hw/pci_devices.h>

/* logging support */
#include <stdarg.h>
#include <sys/slog.h>
#include <sys/slogcodes.h>


#define I2C_CONTROL      0x00
#define I2C_STATUS       0x01
#define I2C_SL_ADDR      0x02
#define I2C_RW_ADDR      0x03
#define I2C_DATA_REG     0x04
#define I2C_CLK_CONTROL1 0x05
#define I2C_CLK_CONTROL2 0x06
#define I2C_EX_CONTROL   0x07

//#define I2C_TIMEOUT       20000
//SCL CLK=400kHz
#define I2C_CLK1_400      0x26
//SCL CLK=100kHz
#define I2C_CLK1_100      0x98
#define I2C_CLK2_VAL      0x81
#define I2C_TX_DONE       0x20
#define I2C_TX_STOP       0x02
#define I2C_NACK_EN       0x01
#define I2C_RX_RDY        0x40

#define I2C_DEF_STATUS    0xFC
#define I2C_DEF_CONTROL   0x00
#define I2C_DEF_BASE_ADDR 0xF100

#define one_ms_in_ticks    1190
#define I2C_BASE_PCICFG     0x000000D4
#define I2C_PINMODE_PCICFG  0x000000CC
#define I2C_PWR_PCICFG      0x000000BC

#define VORTEX_ICNT_PCICFG  0x800000A0
#define VORTEX_ICNT_START   0x498
#define VORTEX_ICNT_STOP    0x499
#define VORTEX_ICNT_VALUE   0x494
#define TICKS_IN_1USEC      600
#define TICKS_IN_10USEC     6000

#define I2C_CHANNEL0        0
#define I2C_CHANNEL1        1

#define I2C_LASTBYTE        1
#define I2C_NOTLASTBYTE     0

#define DEF_TIMEOUT         200


typedef struct {
    unsigned            physbase;

    unsigned            scl_freq;
    i2c_addr_t          slave_addr;

    pthread_mutex_t     lock;
    uint16_t            timeout;
    unsigned            device;
    unsigned            vendor;
} vortex_i2c_dev_t;


// uOps
int v_i2c_slogf(int, const char *, ...);
int vortex_i2c_set_bus_speed(void *, unsigned int, unsigned int *);
int vortex_i2c_version_info(i2c_libversion_t *);
int vortex_i2c_driver_info(void *, i2c_driver_info_t *);
int  vortex_i2c_set_slave_addr(void *, unsigned int, i2c_addrfmt_t);
void vortex_i2c_fini(void *);
int vortex_i2c_parseopts (vortex_i2c_dev_t*, int, char**);
int attach_to_device (unsigned vendor, unsigned device);
void inline i2c_reset( uint16_t ba );
void inline i2c_clr_status( uint16_t ba );
void inline i2c_put_addr( uint16_t ba, uint8_t addr );
uint8_t inline i2c_get_byte( uint16_t ba );
void inline i2c_put_byte( uint16_t ba, uint8_t dat );
void inline i2c_set_stop( uint16_t ba );
void inline i2c_clr_rx( uint16_t ba );
void inline i2c_clr_tx( uint16_t ba );
void inline i2c_set_stop( uint16_t ba );
uint16_t inline i2c_wait_tx( uint16_t ba, uint16_t timeout );
uint16_t inline i2c_wait_rx( uint16_t ba, uint16_t timeout );
uint32_t inline pci_get_cfg_reg( uint32_t addr );
void inline pci_set_cfg_reg( uint32_t addr, uint32_t dat );
uint16_t inline i2c_get_base( uint8_t channel );
i2c_status_t vortex_i2c_send(void *, void *, unsigned int, unsigned int);
void *vortex_i2c_init(int, char **);
i2c_status_t vortex_i2c_recv(void *, void *, unsigned int, unsigned int);

// API functions
uint16_t I2C_Get_Base();
void I2C_Set_Base( uint16_t ba );
void I2C_Power_On( uint8_t channel );
uint16_t I2C_Init( uint8_t channel );       // set base address if not zero 
uint16_t I2C_Start( uint8_t channel, uint8_t addr, uint8_t gen_stop, uint16_t timeout );
uint16_t I2C_ReadByte( uint8_t channel, uint8_t * dat, uint8_t lastbyte, uint16_t timeout );
uint16_t I2C_WriteByte( uint8_t channel, uint8_t dat, uint8_t lastbyte, uint16_t timeout ); 

#endif /* PROTO_H_ */
